@@ -1,11 +1,12 @@
+from . import actuation, contacts
 from .aba import aba
 from .collidable_points import collidable_points_pos_vel
 from .crba import crba
-from .forward_kinematics import forward_kinematics, forward_kinematics_model
+from .forward_kinematics import forward_kinematics_model
 from .jacobian import (
     jacobian,
     jacobian_derivative_full_doubly_left,
     jacobian_full_doubly_left,
 )
+from .kinematic_constraints import compute_constraint_wrenches
 from .rnea import rnea
-from .soft_contacts import SoftContacts, SoftContactsParams
